package com.yfd.platform.aspect;

import cn.hutool.core.util.StrUtil;
import com.yfd.platform.annotation.Idempotent;
import com.yfd.platform.utils.RequestKeyGenerator;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;

import javax.annotation.Resource;
import java.lang.reflect.Method;

/**
 * @Author: zcg
 * @Description: 幂等切面实现
 * @Date: 2024/3/12
 **/
@Aspect
@Configuration
@Order(2)
@Slf4j
public class IdempotentAspect {

    @Resource
    private RedissonClient redissonClient;

    @Around("execution(public * * (..)) && @annotation(com.yfd.platform.annotation.Idempotent)")
    public Object interceptor(ProceedingJoinPoint joinPoint) {
        MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
        Method method = methodSignature.getMethod();
        Idempotent idempotent = method.getAnnotation(Idempotent.class);
        if (StrUtil.isBlank(idempotent.keyPrefix())) {
            throw new RuntimeException("重复提交前缀不能为空");
        }
        //获取自定义key
        final String lockKey = RequestKeyGenerator.getLockKey(joinPoint);
        // 使用Redisson分布式锁的方式判断是否重复提交
        RLock lock = redissonClient.getLock(lockKey);
        boolean isLocked = false;
        try {
            //尝试抢占锁
            isLocked = lock.tryLock();
            //没有拿到锁说明已经有了请求了
            if (!isLocked) {
                throw new RuntimeException(idempotent.message());
            }
            //拿到锁后设置过期时间
            lock.lock(idempotent.timeout(), idempotent.timeUnit());
            try {
                return joinPoint.proceed();
            } catch (Throwable throwable) {
                log.info("系统异常，", throwable);
                throw new RuntimeException("系统异常，" + throwable.getMessage());
            }
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        } finally {
            //释放锁
            if (isLocked && lock.isHeldByCurrentThread()) {
                lock.unlock();
            }
        }
    }
}


