package com.yfd.platform.modules.algorithm.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.algorithm.domain.AlgorithmClassComponent;
import com.yfd.platform.modules.algorithm.domain.AlgorithmDevice;
import com.yfd.platform.modules.algorithm.mapper.AlgorithmClassComponentMapper;
import com.yfd.platform.modules.algorithm.mapper.AlgorithmDeviceMapper;
import com.yfd.platform.modules.algorithm.service.IAlgorithmClassComponentService;
import com.yfd.platform.modules.patroltask.domain.TaskTodo;
import com.yfd.platform.utils.CronScheduleUtils;
import org.quartz.CronExpression;
import org.quartz.TriggerUtils;
import org.quartz.impl.triggers.CronTriggerImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zhengsl
 * @since 2025-05-13
 */
@Service
public class AlgorithmClassComponentServiceImpl extends ServiceImpl<AlgorithmClassComponentMapper,
        AlgorithmClassComponent> implements IAlgorithmClassComponentService {

    @Resource
    private AlgorithmDeviceMapper algorithmDeviceMapper;

    @Resource
    private AlgorithmTaskManager algorithmTaskManager;

    /**********************************
     * 用途说明: 算法关联主设备部件
     * 参数说明 algorithmId 算法分类id
     * 参数说明 page 分页参数
     * 返回值说明: com.baomidou.mybatisplus.extension.plugins.pagination.Page<com.yfd.platform.modules.algorithm.domain
     * .AlgorithmClassComponent>
     ***********************************/
    @Override
    public Page<AlgorithmClassComponent> getAlgorithmClassComponentPage(String algorithmId, String stationCode,
                                                                        Page<AlgorithmClassComponent> page) {
        LambdaQueryWrapper<AlgorithmClassComponent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AlgorithmClassComponent::getAlgorithmId, algorithmId);
        queryWrapper.eq(AlgorithmClassComponent::getStationCode, stationCode);
        return this.page(page, queryWrapper);
    }

    /**********************************
     * 用途说明: 删除算法关联主设备部件
     * 参数说明 id 算法部件id
     * 返回值说明: boolean
     ***********************************/
    @Override
    public boolean deleteAlgorithmClassComponent(String id) {
        AlgorithmClassComponent algorithmClassComponent = this.getById(id);
        String algorithmId = algorithmClassComponent.getAlgorithmId();
        String componentId = algorithmClassComponent.getComponentId();
        if (StrUtil.isNotBlank(algorithmId) && StrUtil.isNotBlank(componentId)) {
            LambdaQueryWrapper<AlgorithmDevice> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(AlgorithmDevice::getAlgorithmId, algorithmId).eq(AlgorithmDevice::getComponentId,
                    componentId);
            algorithmDeviceMapper.delete(queryWrapper);
        }

        return this.removeById(id);
    }

    @Override
    public boolean batchAddAlgorithmClassComponent(List<AlgorithmClassComponent> algorithmClassComponentList) {
        return this.saveOrUpdateBatch(algorithmClassComponentList);
    }

    @Override
    public boolean setAlgorithmClassComponentStatus(String id, String isenable) throws ParseException {
        AlgorithmClassComponent algorithmClassComponent = this.getById(id);

        if (algorithmClassComponent == null) {
            return false;
        }
        if (isenable.equals(algorithmClassComponent.getIsenable())) {
            throw new RuntimeException("当前状态与要设置的状态一致");
        }
        if ("1".equals(isenable)) {
            //TODO 根据cron表达式设置定时任务
            this.createAlgorithmTaskList(algorithmClassComponent);
        }
        if ("0".equals(isenable)) {
            this.deleteAllTasksByComponentId(algorithmClassComponent);
        }
        LambdaUpdateWrapper<AlgorithmClassComponent> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(AlgorithmClassComponent::getId, id).set(AlgorithmClassComponent::getIsenable, isenable);
        return this.update(updateWrapper);
    }

    private void deleteAllTasksByComponentId(AlgorithmClassComponent component) {
        // 调用 Quartz 工具类删除任务
        try {
            algorithmTaskManager.deleteAllTasksByComponentId(component.getId(), component.getStationCode());
        } catch (Exception e) {
            throw new RuntimeException("删除定时任务失败：" + e.getMessage(), e);
        }
    }

    @Override
    public boolean createAlgorithmTaskList(AlgorithmClassComponent algorithmClassComponent) throws ParseException {
        // 判断cron表达式有效性
        if (StrUtil.isBlank(algorithmClassComponent.getCronValue())) {
            return false;
        }
        String cronValue = algorithmClassComponent.getCronValue();
        if (!CronExpression.isValidExpression(cronValue)) {
            throw new RuntimeException("cron表达式格式错误");
        }
        List<Date> fireTimes = CronScheduleUtils.getFireTimes(cronValue, 30);
        // 判断是否有定时任务
        if (fireTimes.isEmpty()) {
            throw new RuntimeException("当前cron表达式没有生成任务");
        }
        if (fireTimes.size() > 180) {
            throw new RuntimeException("当前cron表达式生成任务数量超过最大数量");
        }
        for (Date fireTime : fireTimes) {
            setToDoTask(algorithmClassComponent, DateUtil.formatDateTime(fireTime));
        }
        return true;
    }

    /**
     * 设置待办任务的时间
     * 此方法用于根据提供的算法类组件和触发时间来安排待办任务
     * 它不返回任何值，但可能根据需要更新或创建待办任务
     *
     * @param algorithmClassComponent 算法类组件，代表需要执行的任务的类型或相关信息
     * @param fireTime                任务触发时间，采用字符串形式，具体的格式需要根据上下文或文档确定
     */
    private void setToDoTask(AlgorithmClassComponent algorithmClassComponent, String fireTime) {
        try {
            algorithmTaskManager.addTask(algorithmClassComponent, fireTime);
        } catch (Exception e) {
            throw new RuntimeException("添加定时任务失败：" + e.getMessage(), e);
        }
    }
}
