package com.yfd.platform.modules.algorithm.service.impl;

import com.yfd.platform.modules.algorithm.domain.AlgorithmClassComponent;
import org.quartz.*;
import org.quartz.impl.DirectSchedulerFactory;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.matchers.GroupMatcher;
import org.quartz.simpl.RAMJobStore;
import org.quartz.simpl.SimpleThreadPool;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.*;

/**
 * 算法任务管理器
 */
@Component
public class AlgorithmTaskManager {

    private static final String JOB_NAME_PREFIX = "ALGORITHM_COMPONENT_";
    private static final String JOB_GROUP_PREFIX = "STATION_";

    // 存储每个 stationCode 对应的 Scheduler
    private Map<String, Scheduler> schedulers = new HashMap<>();

    /**
     * 添加定时任务
     *
     * @param component 算法类组件
     * @param fireTime  触发时间（格式：yyyy-MM-dd HH:mm:ss）
     */
    public void addTask(AlgorithmClassComponent component, String fireTime) throws Exception {
        String stationCode = component.getStationCode();
        String jobId = component.getId();

        Scheduler scheduler = getOrCreateScheduler(stationCode);

        // 使用 jobId + fireTime.hashCode() 构建唯一任务 Key
        String uniqueKey = jobId + "_" + fireTime.hashCode();
        JobKey jobKey = JobKey.jobKey(JOB_NAME_PREFIX + uniqueKey, JOB_GROUP_PREFIX + stationCode);
        TriggerKey triggerKey = TriggerKey.triggerKey(JOB_NAME_PREFIX + uniqueKey, JOB_GROUP_PREFIX + stationCode);

        // 如果任务已存在，先删除
        if (scheduler.checkExists(jobKey)) {
            scheduler.pauseJob(jobKey);
            scheduler.unscheduleJob(triggerKey);
            scheduler.deleteJob(jobKey);
        }

        // 创建任务详情
        JobDetail jobDetail = JobBuilder.newJob(AlgorithmTaskJob.class)
                .withIdentity(jobKey)
                .build();

        Date planDate = org.apache.commons.lang3.time.DateUtils.parseDate(fireTime, "yyyy-MM-dd HH:mm:ss");

        // 创建一次性触发器
        Trigger trigger = TriggerBuilder.newTrigger()
                .withIdentity(triggerKey)
                .startAt(planDate)
                .withSchedule(SimpleScheduleBuilder.simpleSchedule()
                        .withRepeatCount(0))
                .build();

        // 传递参数给任务
        jobDetail.getJobDataMap().put("algorithmClassComponent", component);

        // 注册并启动任务
        scheduler.scheduleJob(jobDetail, trigger);
        scheduler.start();
    }

    /**
     * 删除指定ID的所有任务
     *
     * @param id 主设备部件ID
     */
    /**
     * 删除指定算法组件的所有定时任务（基于 jobId 前缀匹配）
     *
     * @param id          主设备部件ID
     * @param stationCode 变电站编码
     */
    public void deleteAllTasksByComponentId(String id, String stationCode) throws Exception {
        Scheduler scheduler = getOrCreateScheduler(stationCode);

        // 构建任务名前缀，如 ALGORITHM_COMPONENT_123_
        String jobNamePrefix = JOB_NAME_PREFIX + id + "_";
        GroupMatcher<JobKey> matcher = GroupMatcher.jobGroupEquals(JOB_GROUP_PREFIX + stationCode);

        for (JobKey jobKey : scheduler.getJobKeys(matcher)) {
            if (jobKey.getName().startsWith(jobNamePrefix)) {
                try {
                    // 获取所有关联触发器
                    List<? extends Trigger> triggers = scheduler.getTriggersOfJob(jobKey);
                    for (Trigger trigger : triggers) {
                        TriggerKey triggerKey = trigger.getKey();
                        // 暂停触发器
                        scheduler.pauseTrigger(triggerKey);
                        // 移除触发器
                        scheduler.unscheduleJob(triggerKey);
                    }
                    scheduler.deleteJob(jobKey); // 删除任务
                    System.out.println("成功删除任务：" + jobKey);

                } catch (Exception e) {
                    System.err.println("删除任务失败：" + jobKey + "，原因：" + e.getMessage());
                }
            }
        }
    }


    /**
     * 获取或创建指定变电站的调度器
     */
    private Scheduler getOrCreateScheduler(String stationCode) throws Exception {
        Scheduler scheduler = schedulers.get(stationCode);
        if (scheduler == null || !scheduler.isStarted()) {
            DirectSchedulerFactory factory = DirectSchedulerFactory.getInstance();
            factory.createScheduler(stationCode, stationCode,
                    new SimpleThreadPool(5, Thread.NORM_PRIORITY),
                    new RAMJobStore());
            scheduler = factory.getScheduler(stationCode);
            schedulers.put(stationCode, scheduler);
        }
        return scheduler;
    }

    /**
     * 关闭所有调度器
     */
    public void shutdownAllSchedulers() throws SchedulerException {
        for (Scheduler scheduler : schedulers.values()) {
            if (scheduler.isStarted()) {
                scheduler.shutdown();
            }
        }
    }
}
